IMPLEMENTATION MODULE MultiSkelHelp;

FROM SYSTEM IMPORT ADDRESS, ADR, SAVEREGS;
FROM QuickDraw IMPORT Rect, EraseRect, TextFont, TextSize, Point;
FROM ResourceManager IMPORT GetResource, ReleaseResource;
FROM MemoryManager IMPORT GetHandleSize, HLock, HUnlock;
FROM MenuManager IMPORT EnableItem, DisableItem, DrawMenuBar;
FROM WindowManager IMPORT WindowPtr, ValidRect, GetNewWindow, DrawGrowIcon,
													CloseWindow;
FROM ControlManager IMPORT ControlHandle, NewControl, scrollBarProc, GetCtlMax,
													 TrackControl, GetCRefCon, DrawControls, HiliteControl,
													 SetCtlValue, GetCtlValue, DisposeControl, SetCRefCon,
													 HideControl, SizeControl, MoveControl, ShowControl,
													 inThumb, inPageUp, inPageDown, inUpButton,
													 inDownButton, TestControl, SetCtlMax;
FROM TextEdit IMPORT TEHandle, TENew, TEInsert, TEDispose, TECalText, TEUpdate,
										 TEScroll, CharsHandle;
FROM MultiSkelGlobals IMPORT helpWindRes, helpWind, helpTextRes, DrawGrowBox, editMenu;
FROM TransSkel IMPORT SkelWindow, WKeyProc, WCloseProc, WIdleProc;

VAR
	teHelp: TEHandle;
	helpScroll: ControlHandle;
	helpLine: INTEGER;
	helpScrollLines: INTEGER;
	halfPage: INTEGER;


PROCEDURE HelpWindHalt;

BEGIN
	TEDispose(teHelp);
	DisposeControl(helpScroll);
	CloseWindow(helpWind);
END HelpWindHalt;


(*
	Scroll to the correct position.  lDelta is the amount by which to CHANGE
	the current scroll setting.
*)

PROCEDURE DoScroll(lDelta: INTEGER);

	VAR
		newLine: INTEGER;

BEGIN
	newLine := helpLine + lDelta;
	IF newLine < 0 THEN newLine := 0; END;
	IF newLine > GetCtlMax(helpScroll) THEN newLine := GetCtlMax(helpScroll); END;
	SetCtlValue(helpScroll, newLine);
	lDelta := (helpLine-newLine) * teHelp^^.lineHeight;
	TEScroll(0, lDelta, teHelp);
	helpLine := newLine;
END DoScroll;


(*
	Filter proc for tracking mousedown in scroll bar.  The part code of the part
	originally hit is stored as the control's reference value.
*)

PROCEDURE TrackScroll(theScroll: ControlHandle; partCode: INTEGER);

	VAR
		lDelta: INTEGER;

BEGIN
	SAVEREGS;
	IF VAL(LONGINT, partCode) = GetCRefCon(theScroll) THEN
		CASE partCode OF
			inUpButton: lDelta := -1;
			|
			inDownButton:	lDelta := 1;
			|
			inPageUp:	lDelta := -halfPage;
			|
			inPageDown:	lDelta := halfPage;
			ELSE (* Do nothing *)
		END;
		DoScroll(lDelta);
	END;
END TrackScroll;


(*
	Handle hits in the scroll bar
*)

PROCEDURE HelpWindMouse(thePt: Point; t: LONGINT; mods: BITSET);

	VAR
		thePart: INTEGER;
		whoCares: INTEGER;

BEGIN
	thePart := TestControl(helpScroll, thePt);
	IF thePart = inThumb THEN
		whoCares := TrackControl(helpScroll, thePt, NIL);
		DoScroll(GetCtlValue(helpScroll) - helpLine);
	ELSIF thePart # 0 THEN
		SetCRefCon(helpScroll, thePart);
		whoCares := TrackControl(helpScroll, thePt, ADR(TrackScroll));
	END;
END HelpWindMouse;


(*
	Update help window.  The update event might be in response to a window
	resizing.  If so, resize the rects and recalc the linestarts of the text.
	To resize the rects, only the right edge of the destrect need be changed.
	The viewRect should be sized to the screen.
*)

PROCEDURE HelpWindUpdate(resized: BOOLEAN);

	VAR
		r: Rect;
		viewLines: INTEGER;

BEGIN
	r := helpWind^.portRect;
	EraseRect(r);
	IF resized THEN
		INC(r.left, 4);
		DEC(r.bottom, 2);
		INC(r.top, 2);
		DEC(r.right, 19);
		teHelp^^.destRect.right := r.right;
		teHelp^^.viewRect := r;
		TECalText(teHelp);
	
	(*
		Move and resize the scroll bar as well.  The ValidRect call is made because
		the HideControl adds the control bounds box to the update region, which
		would generate another update event!  Since everything gets drawn below,
		the ValidRect is used to prevent the extra update.
	*)
	
		viewLines := (r.bottom - r.top) DIV teHelp^^.lineHeight;
		helpScrollLines := teHelp^^.nLines - viewLines;
		halfPage := viewLines DIV 2;
		
		HideControl(helpScroll);
		r := helpScroll^^.contrlRect;
		ValidRect(r);
		r := helpWind^.portRect;
		r.left := r.right - 15;
		DEC(r.bottom, 14);
		DEC(r.top);
		INC(r.right);
		SizeControl(helpScroll, r.right - r.left, r.bottom - r.top);
		MoveControl(helpScroll, r.left, r.top);
		SetCtlMax(helpScroll, helpScrollLines);
		(*
			Strange expression following, but the distribution copy in C had:
				if (helpScrollLines <= 0)
					HiliteControl(helpScroll, (helpScrollLines > 0 ? 0 : 255));
			This left controls in funny state  changed first test.
		*)
		IF helpScrollLines >= 0 THEN 
			IF helpScrollLines > 0 THEN HiliteControl(helpScroll, 0);
			ELSE HiliteControl(helpScroll, 255); END;
		END;
		ShowControl(helpScroll);
		IF GetCtlValue(helpScroll) > helpScrollLines THEN
			DoScroll(GetCtlValue(helpScroll) - helpScrollLines);
		END;
	END;
	DrawGrowBox(helpWind);
	DrawControls(helpWind);
	r := teHelp^^.viewRect;
	TEUpdate(r, teHelp);
END HelpWindUpdate;


(*
	When the window comes active, disable the Edit menu and highlight the scroll
	bar if tere are any lines not visible in the content region.  When the
	window is deactivated, enable the Edit menu and unhighlight the scroll bar.
*)

PROCEDURE HelpWindActivate(active: BOOLEAN);

BEGIN
	DrawGrowBox(helpWind);
	IF active THEN
		DisableItem(editMenu, 0);
		IF helpScrollLines > 0 THEN HiliteControl(helpScroll, 0);
		ELSE HiliteControl(helpScroll, 255); END;
	ELSE
		EnableItem(editMenu, 0);
		HiliteControl(helpScroll, 255);
	END;
	DrawMenuBar;
END HelpWindActivate;


PROCEDURE HelpWindInit;

	VAR
		r: Rect;
		textHandle: ADDRESS;
		viewLines: INTEGER;
		aChHdl: CharsHandle;

BEGIN
	helpWind := GetNewWindow(helpWindRes, VAL(ADDRESS, NIL), VAL(WindowPtr, -1D));
	SkelWindow(	helpWind,
							HelpWindMouse,					(* Mouse click handler *)
							VAL(WKeyProc, NIL),			(* Key click handler *)
							HelpWindUpdate,					(* Update event handler *)
							HelpWindActivate,				(* Activate/deactivate event handler *)
							VAL(WCloseProc, NIL),		(* No close handler *)
							HelpWindHalt,						(* Clobber proc *)
							VAL(WIdleProc, NIL),		(* No idle proc for the help window *)
							TRUE);									(* Doesn't really matter since no idle *)
	
	TextFont(0);
	TextSize(0);
	
	r := helpWind^.portRect;
	INC(r.left, 4);
	DEC(r.bottom, 2);
	INC(r.top, 2);
	DEC(r.right, 19);
	teHelp := TENew(r, r);
	textHandle := GetResource("TEXT", helpTextRes);	(* Read the help text *)
	aChHdl := VAL(CharsHandle, textHandle);
	HLock(textHandle);
	TEInsert(aChHdl^, GetHandleSize(textHandle), teHelp);
	HUnlock(textHandle);
	ReleaseResource(textHandle);	(* Done with it - Ciao *)
	
(*
	Now, figure out how many lines will fit in the window and how many will not.
	Determine the number of lines in half a window for use in tracking clicks in
	the page up and page down regions of the scroll bar.  Then create the scroll
	bar.  Make sure the borders overlap the window frame and grow box frame.
*)
	viewLines := (r.bottom - r.top) DIV teHelp^^.lineHeight;
	helpScrollLines := teHelp^^.nLines - viewLines;
	halfPage := viewLines DIV 2;
	helpLine := 0;
	r := helpWind^.portRect;
	r.left := r.right - 15;
	DEC(r.bottom, 14);
	DEC(r.top);
	INC(r.right);
	
	(*
		Build the scroll bar.  Don't need to bother testing whether to highlight it
		or not, since that will be done in response to the activate event.
	*)
	helpScroll := NewControl(helpWind, r, '', TRUE, helpLine, 0, helpScrollLines,
													 scrollBarProc, 0D);
	
	(*
		GetNewWindow generates update event for the entire portrect.  Cancel it,
		since everything has been drawn already, except for the grow box which
		will be drawn in response to the activate event.
	*)
	ValidRect(helpWind^.portRect);
END HelpWindInit;

END MultiSkelHelp.

